import optparse

#------------------------------------------------------------------------------
def error(text):
  sys.stderr.write('Error: %s\n' % text)
  sys.exit(-1)

#------------------------------------------------------------------------------
def crc32_tab_gen():
  res = []

  for i in range(256):
    value = i

    for j in range(8):
      if value & 1:
        value = (value >> 1) ^ 0xedb88320
      else:
        value = value >> 1

    res += [value]

  return res

#------------------------------------------------------------------------------
def crc32(crc32_tab, data):
  crc = 0xffffffff

  for d in data:
    crc = crc32_tab[(crc ^ d) & 0xff] ^ (crc >> 8)

  return crc

#------------------------------------------------------------------------------
def main():
  parser = optparse.OptionParser(usage = 'usage: %prog [options]')
  parser.add_option('-i', '--input', dest='input', help='input file', metavar='INPUT')
  parser.add_option('-o', '--output', dest='output', help='output file', metavar='OUTPUT')
  (options, args) = parser.parse_args()

  if options.input is None or options.output is None:
    error('input and output file names are required (see -h)')

  app = [ord(x) for x in open(options.input, 'rb').read()]

  while (len(app) + 4) % 64 > 0:
    app += [0xff]

  app[28] = ((len(app) + 4) >> 0) & 0xff
  app[29] = ((len(app) + 4) >> 8) & 0xff
  app[30] = ((len(app) + 4) >> 16) & 0xff
  app[31] = ((len(app) + 4) >> 24) & 0xff

  crc32_tab = crc32_tab_gen()
  crc = crc32(crc32_tab, app)

  app += [(crc >> 0) & 0xff]
  app += [(crc >> 8) & 0xff]
  app += [(crc >> 16) & 0xff]
  app += [(crc >> 24) & 0xff]

  f = open(options.output, 'wb')
  f.write(''.join(chr(i) for i in app))
  f.close()

#------------------------------------------------------------------------------
main()
